#include "robot.h"
#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>

static int width;
static int height;
static unsigned char* field;

void initialize(int w, int h)
{
  width = w;
  height = h;
  field = (unsigned char*)malloc(w*h);
  
  for(int i = 0; i < height; i++)
  {
    for(int j = 0; j < width; j++)
    {
      field[i*width+j] = (unsigned char)(rand()%4);
    }
  }
}

void finalize(void)
{
  free(field);
}

static int x = 0;
static int y = 0;
static int inventory = 0;

bool execute(Command c)
{  
  switch(c.op)
  {
  case left:
    x -= c.count;
    if(x < 0)
    {
      x = 0;
      return false;
    }
    break;
  case right:
    x += c.count;
    if(x >= width)
    {
      x = width-1;
      return false;
    }    
    break;
  case up:
    y -= c.count;
    if(y < 0)
    {
      y = 0;
      return false;
    }
    break;      
  case down:
    y += c.count;
    if(y >= height)
    {
      y = height-1;
      return false;
    }    
    break;
  case fetch:
    {
      int z = field[y*width+x];
      z -= c.count;
      bool err = false;
      if(z < 0)
      {
        z = 0;
        err = true;
      }
      inventory += field[y*width+x] - z;
      field[y*width+x] = z;
      if(err)
      {
        return false;
      }
    }
    break;
  case put:
    {
      int z = inventory;
      z -= c.count;
      bool err = false;
      if(z < 0)
      {
        z = 0;
        err = true;
      }
      field[y*width+x] += inventory - z;
      inventory = z;
      if(err)
      {
        return false;
      }
    }
    break;
  default:
    return false;
  }
  
  return true;
}

void print(void)
{
  for(int i = -1; i < height; i++)
  {
    for(int j = -1; j < width; j++)
    {
      if(j == -1)
      {
        if(i == y)
        {
          putchar('>');
        }
        else
        {  
          putchar(' ');
        }        
      }
      else if(i == -1)
      {
        if(j == x)
        {
          putchar('V');
        }
        else
        {  
          putchar(' ');
        }
      }
      else
      {
        putchar('0' + field[i*width+j]);
      }
    }
    putchar('\n');
  }
}

int count(void)
{
  return field[y*width+x];
}

int getWidth(void)
{
  return width;  
}

int getHeight(void)
{
  return height;
}